"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const injection_1 = require("injection");
const constant_1 = require("../constant");
var RouteParamTypes;
(function (RouteParamTypes) {
    RouteParamTypes[RouteParamTypes["QUERY"] = 0] = "QUERY";
    RouteParamTypes[RouteParamTypes["BODY"] = 1] = "BODY";
    RouteParamTypes[RouteParamTypes["PARAM"] = 2] = "PARAM";
    RouteParamTypes[RouteParamTypes["HEADERS"] = 3] = "HEADERS";
    RouteParamTypes[RouteParamTypes["SESSION"] = 4] = "SESSION";
    RouteParamTypes[RouteParamTypes["FILESTREAM"] = 5] = "FILESTREAM";
    RouteParamTypes[RouteParamTypes["FILESSTREAM"] = 6] = "FILESSTREAM";
    RouteParamTypes[RouteParamTypes["NEXT"] = 7] = "NEXT";
})(RouteParamTypes = exports.RouteParamTypes || (exports.RouteParamTypes = {}));
exports.extractValue = function extractValue(key, data) {
    return async function (ctx, next) {
        switch (key) {
            case RouteParamTypes.NEXT:
                return next;
            case RouteParamTypes.BODY:
                return data && ctx.request.body ? ctx.request.body[data] : ctx.request.body;
            case RouteParamTypes.PARAM:
                return data ? ctx.params[data] : ctx.params;
            case RouteParamTypes.QUERY:
                return data ? ctx.query[data] : ctx.query;
            case RouteParamTypes.HEADERS:
                return data ? ctx.headers[data] : ctx.headers;
            case RouteParamTypes.SESSION:
                return ctx.session;
            case RouteParamTypes.FILESTREAM:
                return ctx.getFileStream && ctx.getFileStream(data);
            case RouteParamTypes.FILESSTREAM:
                return ctx.multipart && ctx.multipart(data);
            default:
                return null;
        }
    };
};
const createParamMapping = function (type) {
    return (data) => (target, key, index) => {
        injection_1.attachMethodDataToClass(constant_1.WEB_ROUTER_PARAM_KEY, {
            index,
            type,
            data,
            extractValue: exports.extractValue(type, data)
        }, target, key);
    };
};
exports.session = () => createParamMapping(RouteParamTypes.SESSION)();
exports.body = (property) => createParamMapping(RouteParamTypes.BODY)(property);
exports.query = (property) => createParamMapping(RouteParamTypes.QUERY)(property);
exports.param = (property) => createParamMapping(RouteParamTypes.PARAM)(property);
exports.headers = (property) => createParamMapping(RouteParamTypes.HEADERS)(property);
exports.file = (property) => createParamMapping(RouteParamTypes.FILESTREAM)(property);
exports.files = (property) => createParamMapping(RouteParamTypes.FILESSTREAM)(property);
//# sourceMappingURL=paramMapping.js.map