import { KoaMiddlewareParamArray } from '../interface';
export interface RouterOption {
    path?: string;
    requestMethod: string;
    routerName?: string;
    method: string;
    middleware?: KoaMiddlewareParamArray;
}
export declare const RequestMethod: {
    GET: string;
    POST: string;
    PUT: string;
    DELETE: string;
    PATCH: string;
    ALL: string;
    OPTIONS: string;
    HEAD: string;
};
declare const PATH_METADATA = "PATH_METADATA";
declare const METHOD_METADATA = "METHOD_METADATA";
declare const ROUTER_NAME_METADATA = "ROUTER_NAME_METADATA";
declare const ROUTER_MIDDLEWARE = "ROUTER_MIDDLEWARE";
export interface RequestMappingMetadata {
    [PATH_METADATA]?: string;
    [METHOD_METADATA]: string;
    [ROUTER_NAME_METADATA]?: string;
    [ROUTER_MIDDLEWARE]?: KoaMiddlewareParamArray;
}
export declare const RequestMapping: (metadata?: RequestMappingMetadata) => MethodDecorator;
/**
 * Routes HTTP POST requests to the specified path.
 */
export declare const post: (path?: string, routerOptions?: {
    routerName?: string;
    middleware?: (string | import("../interface").KoaMiddleware<any>)[];
}) => MethodDecorator;
/**
 * Routes HTTP GET requests to the specified path.
 */
export declare const get: (path?: string, routerOptions?: {
    routerName?: string;
    middleware?: (string | import("../interface").KoaMiddleware<any>)[];
}) => MethodDecorator;
/**
 * Routes HTTP DELETE requests to the specified path.
 */
export declare const del: (path?: string, routerOptions?: {
    routerName?: string;
    middleware?: (string | import("../interface").KoaMiddleware<any>)[];
}) => MethodDecorator;
/**
 * Routes HTTP PUT requests to the specified path.
 */
export declare const put: (path?: string, routerOptions?: {
    routerName?: string;
    middleware?: (string | import("../interface").KoaMiddleware<any>)[];
}) => MethodDecorator;
/**
 * Routes HTTP PATCH requests to the specified path.
 */
export declare const patch: (path?: string, routerOptions?: {
    routerName?: string;
    middleware?: (string | import("../interface").KoaMiddleware<any>)[];
}) => MethodDecorator;
/**
 * Routes HTTP OPTIONS requests to the specified path.
 */
export declare const options: (path?: string, routerOptions?: {
    routerName?: string;
    middleware?: (string | import("../interface").KoaMiddleware<any>)[];
}) => MethodDecorator;
/**
 * Routes HTTP HEAD requests to the specified path.
 */
export declare const head: (path?: string, routerOptions?: {
    routerName?: string;
    middleware?: (string | import("../interface").KoaMiddleware<any>)[];
}) => MethodDecorator;
/**
 * Routes all HTTP requests to the specified path.
 */
export declare const all: (path?: string, routerOptions?: {
    routerName?: string;
    middleware?: (string | import("../interface").KoaMiddleware<any>)[];
}) => MethodDecorator;
export {};
