"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * 'HEAD', 'OPTIONS', 'GET', 'PUT', 'PATCH', 'POST', 'DELETE' 封装
 */
const injection_1 = require("injection");
const constant_1 = require("../constant");
exports.RequestMethod = {
    GET: 'get',
    POST: 'post',
    PUT: 'put',
    DELETE: 'delete',
    PATCH: 'patch',
    ALL: 'all',
    OPTIONS: 'options',
    HEAD: 'head',
};
const PATH_METADATA = 'PATH_METADATA';
const METHOD_METADATA = 'METHOD_METADATA';
const ROUTER_NAME_METADATA = 'ROUTER_NAME_METADATA';
const ROUTER_MIDDLEWARE = 'ROUTER_MIDDLEWARE';
const defaultMetadata = {
    [PATH_METADATA]: '/',
    [METHOD_METADATA]: exports.RequestMethod.GET,
    [ROUTER_NAME_METADATA]: null,
    [ROUTER_MIDDLEWARE]: []
};
exports.RequestMapping = (metadata = defaultMetadata) => {
    const path = metadata[PATH_METADATA] || '/';
    const requestMethod = metadata[METHOD_METADATA] || exports.RequestMethod.GET;
    const routerName = metadata[ROUTER_NAME_METADATA];
    const middleware = metadata[ROUTER_MIDDLEWARE];
    return (target, key, descriptor) => {
        injection_1.attachClassMetadata(constant_1.WEB_ROUTER_KEY, {
            path,
            requestMethod,
            routerName,
            method: key,
            middleware
        }, target);
        return descriptor;
    };
};
const createMappingDecorator = (method) => (path, routerOptions = { middleware: [] }) => {
    return exports.RequestMapping({
        [PATH_METADATA]: path,
        [METHOD_METADATA]: method,
        [ROUTER_NAME_METADATA]: routerOptions.routerName,
        [ROUTER_MIDDLEWARE]: routerOptions.middleware,
    });
};
/**
 * Routes HTTP POST requests to the specified path.
 */
exports.post = createMappingDecorator(exports.RequestMethod.POST);
/**
 * Routes HTTP GET requests to the specified path.
 */
exports.get = createMappingDecorator(exports.RequestMethod.GET);
/**
 * Routes HTTP DELETE requests to the specified path.
 */
exports.del = createMappingDecorator(exports.RequestMethod.DELETE);
/**
 * Routes HTTP PUT requests to the specified path.
 */
exports.put = createMappingDecorator(exports.RequestMethod.PUT);
/**
 * Routes HTTP PATCH requests to the specified path.
 */
exports.patch = createMappingDecorator(exports.RequestMethod.PATCH);
/**
 * Routes HTTP OPTIONS requests to the specified path.
 */
exports.options = createMappingDecorator(exports.RequestMethod.OPTIONS);
/**
 * Routes HTTP HEAD requests to the specified path.
 */
exports.head = createMappingDecorator(exports.RequestMethod.HEAD);
/**
 * Routes all HTTP requests to the specified path.
 */
exports.all = createMappingDecorator(exports.RequestMethod.ALL);
//# sourceMappingURL=requestMapping.js.map